/* 
 * "ESCBOXF.H"
 * 
 * Part of The Escotia Playstation Library 
 *
 * Functions to make programming a chore not a pleasure
 *
 * Author:	ScoTT Campbell
 * Date:	10/9/98
 *
*/

#ifndef __ESCBOXFHEADER
#define __ESCBOXFHEADER



/*****************************/
/* Start of GsBOXF functions */ 
/*****************************/

void InitBoxF(GsBOXF* boxPtr, int xPosition, int yPosition, int width, int height);
/* Initialisation of boxes in one easy function											*/
/* xPosition and yPosition are the display position for the top left corner of the box	*/
/* and can be changed after initialisation using MoveBox()								*/
/* width and height specify the size of the box in pixels, and can be changed after		*/
/* initialisation using SetBoxSize()													*/
/* The box's attributes are set to a default of no transparency and display enabled		*/
/* and can be changed with the function SetBoxAttributes()								*/
/* The box's colour is set to a default of white, and can be changed by SetBoxColour()	*/


void SetBoxFPosition(GsBOXF* boxPtr, int xPosition, int yPosition);
/* Box pointed to by boxPtr has display position changed to (xPosition, yPosition)		*/


void SetBoxFXPosition(GsBOXF* boxPtr, int xPosition);
/* Box pointed to by boxPtr has display X position changed to xPosition					*/


void SetBoxFYPosition(GsBOXF* boxPtr, int yPosition);
/* Box pointed to by boxPtr has display Y position changed to yPosition					*/


void MoveBoxF(GsBOXF* boxPtr, int xAmount, int yAmount);
/* Box pointed to by boxPtr has display position adjusted by (xAmount, yAmount)			*/


void SetBoxFSize(GsBOXF* boxPtr, unsigned short int width, unsigned short int height);
/* Box pointed to by boxPtr has size set to width and height							*/


void SetBoxFWidth(GsBOXF* boxPtr, unsigned short int width);
/* Box pointed to by boxPtr has it's width set to width									*/


void SetBoxFHeight(GsBOXF* boxPtr, unsigned short int height);
/* Box pointed to by boxPtr has its' height set to height								*/


int AdjustBoxFHeight(GsBOXF* boxPtr, int amount);
/* Box pointed to by boxPtr has height changed by amount								*/
/* If amount is negative and greater than it's height then height is set to 0 and the	*/
/* function will return 1; otherwise the function returns 0								*/


int AdjustBoxFWidth(GsBOXF* boxPtr, int amount);
/* Box pointed to by boxPtr has width changed by amount									*/
/* If amount is negative and greater than it's width then width is set to 0 and the		*/
/* function will return 1; otherwise the function returns 0								*/


void SetBoxFAttributes(GsBOXF* boxPtr, unsigned long int attributes);
/* Box pointed to by boxPtr has attributes set to value specified by attributes			*/  


void SetBoxFColour(GsBOXF* boxPtr, int red, int green, int blue);
/* Box pointed to by boxPtr has colour set to red, green, blue							*/


void SetBoxFColourVector(GsBOXF* boxPtr, CVECTOR* colourVectorPtr);
/* Box pointed to by boxPtr has colour set to value specified by *colourVectorPtr		*/


/***************************/
/* End of GsBOXF functions */ 
/***************************/

#endif